{                     Propeller Remote Control slave by Nico Hattink            
                       ********************************************        }
VAR  long cog,flags[16]                    ' flags are shared with master propeller

PUB start : okay |i,point                 ' Start remote control slave - starts a cog 
    stop                                  ' returns false if no cog available 
    okay := cog := cognew(@strt, @flags) + 1
    repeat i from 0 to 8
     flags[i]~~

PUB stop                                    ' Stop remote control master - frees a cog
    if cog
       cogstop(cog~ - 1)

PUB SENDMES(string_address)                 'send a string to master VGA
         flags[7]:=string_address          
                                       
PUB trace(point,value)|i                       ' pass tracepoint, deposit value   
         i:=point<<1                             'update pass counter
         flags[i]++
         flags[i+1]:=value                      'deposit a value              

PUB break(point,value)                        'breakpoint
          flags[9]:=point                     'set breakpoint number and deposit value
          flags[10]:=value
          repeat while flags[9]               'wait for release

DAT           org  
strt          mov     dira,copymask          ' debug, show bit read action
              mov     fladr,par              ' save address of flags 
wait          test    cmask,ina   wz         ' test command line low
       if_nz  jmp     #wait                  '
              call    #recbyte               ' get command byte
              mov     slvid,rec              ' isolate slave-id  
              shr     slvid,#4
              and     slvid,#$F             
              cmp     slvid,my_id  wz        ' test if this slave adressed 
       if_nz  jmp     #wait                  ' not me
              and     rec,#$F                ' isolate command code 
              add     rec,#jumps-1           ' compute address in table
              jmp     rec                    ' jump via table 
'-------------------------------------------------------------------
jumps         jmp     #DO_RLONG               '1
              jmp     #DO_WLONG               '2
              jmp     #DO_RPIN               '3
              jmp     #DO_WPIN               '4
              jmp     #DO_RFLAG              '5
              jmp     #DO_WFLAG              '6
              jmp     #DO_STS                '7   request status
              jmp     #DO_RBYTE              '8   not used yet
              jmp     #DO_WBYTE              '9   not used yet
'-------------------------------------------------------
DO_RBYTE      call    #recpar
              rdbyte  send,param
              call    #sendbyte
              jmp     #wait
'-----------------------------------------------------
DO_WBYTE      call    #recpar
              mov     par1,param
              call    #recbyte
              wrbyte  rec,par1
'---------------------------------------------------------
DO_STS        call    #sendbyte
              jmp     #wait
'-------------------------------------------------------
DO_RLONG      call    #recpar
              rdlong  param,param                'read long slave hub memory
              call    #sendpar                   'send to master
              jmp     #wait
'-------------------------------------------------------
DO_WLONG      call    #recpar
              mov     par1,param
              call    #recpar                    'get data and
              wrlong  param,par1                 'write into slave hub memory
              jmp     #wait                      'Go wait for next command
'---------------------------------------------------------
DO_WPIN       call    #recpar
              mov     param,par1
              call    #recpar                    'get data to output to slave pins
              mov     dira,par1                  'mask for output
              mov     outa,param                 'output to pins 
              jmp     #wait                      'Go wait for next command
'--------------------------------------------------------
DO_RPIN       mov     param,ina                 'get actual pin states
              call    #sendpar                     'and return to master
              jmp     #wait
 '--------------------------------------------------------
DO_WFLAG      call    #recbyte
              mov     par1,rec
              call    #recpar                    'get flag data
              shl     par1,#2                    'compute flag address
              add     par1,fladr
              wrlong  param,par1                 'write fla
              jmp     #wait                      'Go wait for next command
'---------------------------------------------------------
DO_RFLAG      call    #recbyte
              shl     rec,#2                    'compute flag address
              add     rec,fladr
              rdlong  temp,rec                  'get flag  data
              mov     param,temp
              call    #sendpar                   'and send to master
              jmp     #wait
'--------------------------------------------------------------
sendbyte      andn  outa,dmask                 'set dataline low
              or    dira,dmask                 'and switch to output
delay         mov   dcount,#20                  'some delay, long startbit
              djnz  dcount,#delay+1
              mov   count,#8                   'prepare for 8 bits
              or    outa,dmask                 'high data signals start of transfer
nextbit       test  send,dmask     wz          'test bit
              muxnz outa,dmask                 'output bit,MSB first
              rol   send,#1
              nop                              '5 ticks per bit, 4Mbit/sec
              djnz  count,#nextbit
              xor   dira,dmask                 'back to input
sendbyte_ret  ret
'--------------------------------------------------------
recbyte       mov     tcount,#100
              test    dmask,ina   wz            'wait for datapin low
       if_z   jmp     #process                  'yes
              djnz    tcount,#recbyte+1          'no, test if time-out
              jmp     #recbyte_ret               '
process       mov     count,#8                  'prepare for 8 bits
              waitpeq dmask,dmask               'wait for DPin high,start transfer
              nop
              mov     rec,#0                    'clear result
nextbit2      shl     rec,#1        
              test    dmask,ina   wz            'test bit, MSB first
      if_nz   add     rec,#1                    'add bit result
              xor     outa,copymask             'debug, show bit read action
              djnz    count,#nextbit2           'done?
recbyte_ret   ret 
'-------------------------------------------------
sendpar       mov    bcount,#4                     '4 bytes
              rol    param,#8                     'MSByte first
              mov    send,param                   '
              call   #sendbyte                    'send
              djnz   bcount,#sendpar+1              'next byte
sendpar_ret   ret
'------------------------------------------------------
recpar        mov    bcount,#4
              shl    param,#8 
              call   #recbyte
              add    param,rec
              djnz   bcount,#recpar+ 1
recpar_ret    ret
'--------------------------------------------------
dmask         long     $00_00_00_80              'mask for data A7
cmask         long     $00_00_00_40              'mask for command clock on A6
copymask      long     1                         'debug, output bitread toggle on A0
my_id         long     1                         'this slave id
'----------------------------------------------  
count         res      1                         'general counter
bcount        res      1                         'byte counter
tcount        res      1                         'time-out counter
dcount        res      1                         'delay counter
fladr         res      1                         'address of flags 0 - 3                                   
rec           res      1                         'received databyte  
send          res      1                         'databyte to send
temp          res      1
slvid         res      1                         'slave id as adressed by master
cmd           res      1                         'command received from master
par1          res      1                         'first parameter received from master
par2          res      1                         'second param
param         res      1                         '